# Generated from tools/antlr/GnosisEPL.g4 by ANTLR 4.8
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .GnosisEPLParser import GnosisEPLParser
else:
    from GnosisEPLParser import GnosisEPLParser

# This class defines a complete listener for a parse tree produced by GnosisEPLParser.
class GnosisEPLListener(ParseTreeListener):

    # Enter a parse tree produced by GnosisEPLParser#alphanumeric.
    def enterAlphanumeric(self, ctx:GnosisEPLParser.AlphanumericContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#alphanumeric.
    def exitAlphanumeric(self, ctx:GnosisEPLParser.AlphanumericContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#comparison_operator.
    def enterComparison_operator(self, ctx:GnosisEPLParser.Comparison_operatorContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#comparison_operator.
    def exitComparison_operator(self, ctx:GnosisEPLParser.Comparison_operatorContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#query.
    def enterQuery(self, ctx:GnosisEPLParser.QueryContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#query.
    def exitQuery(self, ctx:GnosisEPLParser.QueryContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#subcription.
    def enterSubcription(self, ctx:GnosisEPLParser.SubcriptionContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#subcription.
    def exitSubcription(self, ctx:GnosisEPLParser.SubcriptionContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#query_name.
    def enterQuery_name(self, ctx:GnosisEPLParser.Query_nameContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#query_name.
    def exitQuery_name(self, ctx:GnosisEPLParser.Query_nameContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#output_list.
    def enterOutput_list(self, ctx:GnosisEPLParser.Output_listContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#output_list.
    def exitOutput_list(self, ctx:GnosisEPLParser.Output_listContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#output.
    def enterOutput(self, ctx:GnosisEPLParser.OutputContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#output.
    def exitOutput(self, ctx:GnosisEPLParser.OutputContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#separator.
    def enterSeparator(self, ctx:GnosisEPLParser.SeparatorContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#separator.
    def exitSeparator(self, ctx:GnosisEPLParser.SeparatorContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#content.
    def enterContent(self, ctx:GnosisEPLParser.ContentContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#content.
    def exitContent(self, ctx:GnosisEPLParser.ContentContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#content_service.
    def enterContent_service(self, ctx:GnosisEPLParser.Content_serviceContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#content_service.
    def exitContent_service(self, ctx:GnosisEPLParser.Content_serviceContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#match_clause.
    def enterMatch_clause(self, ctx:GnosisEPLParser.Match_clauseContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#match_clause.
    def exitMatch_clause(self, ctx:GnosisEPLParser.Match_clauseContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#relationship.
    def enterRelationship(self, ctx:GnosisEPLParser.RelationshipContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#relationship.
    def exitRelationship(self, ctx:GnosisEPLParser.RelationshipContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#relationship_ref_with_class.
    def enterRelationship_ref_with_class(self, ctx:GnosisEPLParser.Relationship_ref_with_classContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#relationship_ref_with_class.
    def exitRelationship_ref_with_class(self, ctx:GnosisEPLParser.Relationship_ref_with_classContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#relationship_ref_middle.
    def enterRelationship_ref_middle(self, ctx:GnosisEPLParser.Relationship_ref_middleContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#relationship_ref_middle.
    def exitRelationship_ref_middle(self, ctx:GnosisEPLParser.Relationship_ref_middleContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#relationship_type.
    def enterRelationship_type(self, ctx:GnosisEPLParser.Relationship_typeContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#relationship_type.
    def exitRelationship_type(self, ctx:GnosisEPLParser.Relationship_typeContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#left_object.
    def enterLeft_object(self, ctx:GnosisEPLParser.Left_objectContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#left_object.
    def exitLeft_object(self, ctx:GnosisEPLParser.Left_objectContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#right_object.
    def enterRight_object(self, ctx:GnosisEPLParser.Right_objectContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#right_object.
    def exitRight_object(self, ctx:GnosisEPLParser.Right_objectContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#object_ref_with_class.
    def enterObject_ref_with_class(self, ctx:GnosisEPLParser.Object_ref_with_classContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#object_ref_with_class.
    def exitObject_ref_with_class(self, ctx:GnosisEPLParser.Object_ref_with_classContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#object_class.
    def enterObject_class(self, ctx:GnosisEPLParser.Object_classContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#object_class.
    def exitObject_class(self, ctx:GnosisEPLParser.Object_classContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#object_ref.
    def enterObject_ref(self, ctx:GnosisEPLParser.Object_refContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#object_ref.
    def exitObject_ref(self, ctx:GnosisEPLParser.Object_refContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#attributes.
    def enterAttributes(self, ctx:GnosisEPLParser.AttributesContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#attributes.
    def exitAttributes(self, ctx:GnosisEPLParser.AttributesContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#attribute.
    def enterAttribute(self, ctx:GnosisEPLParser.AttributeContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#attribute.
    def exitAttribute(self, ctx:GnosisEPLParser.AttributeContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#attribute_name.
    def enterAttribute_name(self, ctx:GnosisEPLParser.Attribute_nameContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#attribute_name.
    def exitAttribute_name(self, ctx:GnosisEPLParser.Attribute_nameContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#attribute_value.
    def enterAttribute_value(self, ctx:GnosisEPLParser.Attribute_valueContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#attribute_value.
    def exitAttribute_value(self, ctx:GnosisEPLParser.Attribute_valueContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#attribute_value_str.
    def enterAttribute_value_str(self, ctx:GnosisEPLParser.Attribute_value_strContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#attribute_value_str.
    def exitAttribute_value_str(self, ctx:GnosisEPLParser.Attribute_value_strContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#attribute_value_str_inner.
    def enterAttribute_value_str_inner(self, ctx:GnosisEPLParser.Attribute_value_str_innerContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#attribute_value_str_inner.
    def exitAttribute_value_str_inner(self, ctx:GnosisEPLParser.Attribute_value_str_innerContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#attribute_value_num.
    def enterAttribute_value_num(self, ctx:GnosisEPLParser.Attribute_value_numContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#attribute_value_num.
    def exitAttribute_value_num(self, ctx:GnosisEPLParser.Attribute_value_numContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#logical_operator.
    def enterLogical_operator(self, ctx:GnosisEPLParser.Logical_operatorContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#logical_operator.
    def exitLogical_operator(self, ctx:GnosisEPLParser.Logical_operatorContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#where_clause.
    def enterWhere_clause(self, ctx:GnosisEPLParser.Where_clauseContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#where_clause.
    def exitWhere_clause(self, ctx:GnosisEPLParser.Where_clauseContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#where_logical_op_set.
    def enterWhere_logical_op_set(self, ctx:GnosisEPLParser.Where_logical_op_setContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#where_logical_op_set.
    def exitWhere_logical_op_set(self, ctx:GnosisEPLParser.Where_logical_op_setContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#where_attribute.
    def enterWhere_attribute(self, ctx:GnosisEPLParser.Where_attributeContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#where_attribute.
    def exitWhere_attribute(self, ctx:GnosisEPLParser.Where_attributeContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#where_attribute_name.
    def enterWhere_attribute_name(self, ctx:GnosisEPLParser.Where_attribute_nameContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#where_attribute_name.
    def exitWhere_attribute_name(self, ctx:GnosisEPLParser.Where_attribute_nameContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#publisher_list.
    def enterPublisher_list(self, ctx:GnosisEPLParser.Publisher_listContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#publisher_list.
    def exitPublisher_list(self, ctx:GnosisEPLParser.Publisher_listContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#publisher.
    def enterPublisher(self, ctx:GnosisEPLParser.PublisherContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#publisher.
    def exitPublisher(self, ctx:GnosisEPLParser.PublisherContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#window.
    def enterWindow(self, ctx:GnosisEPLParser.WindowContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#window.
    def exitWindow(self, ctx:GnosisEPLParser.WindowContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#window_type.
    def enterWindow_type(self, ctx:GnosisEPLParser.Window_typeContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#window_type.
    def exitWindow_type(self, ctx:GnosisEPLParser.Window_typeContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#window_arg_list.
    def enterWindow_arg_list(self, ctx:GnosisEPLParser.Window_arg_listContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#window_arg_list.
    def exitWindow_arg_list(self, ctx:GnosisEPLParser.Window_arg_listContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#window_arg.
    def enterWindow_arg(self, ctx:GnosisEPLParser.Window_argContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#window_arg.
    def exitWindow_arg(self, ctx:GnosisEPLParser.Window_argContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#qos_metric_list.
    def enterQos_metric_list(self, ctx:GnosisEPLParser.Qos_metric_listContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#qos_metric_list.
    def exitQos_metric_list(self, ctx:GnosisEPLParser.Qos_metric_listContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#qos_metric.
    def enterQos_metric(self, ctx:GnosisEPLParser.Qos_metricContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#qos_metric.
    def exitQos_metric(self, ctx:GnosisEPLParser.Qos_metricContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#qos_metric_name.
    def enterQos_metric_name(self, ctx:GnosisEPLParser.Qos_metric_nameContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#qos_metric_name.
    def exitQos_metric_name(self, ctx:GnosisEPLParser.Qos_metric_nameContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#qos_metric_value.
    def enterQos_metric_value(self, ctx:GnosisEPLParser.Qos_metric_valueContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#qos_metric_value.
    def exitQos_metric_value(self, ctx:GnosisEPLParser.Qos_metric_valueContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#return_clause.
    def enterReturn_clause(self, ctx:GnosisEPLParser.Return_clauseContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#return_clause.
    def exitReturn_clause(self, ctx:GnosisEPLParser.Return_clauseContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#node_list.
    def enterNode_list(self, ctx:GnosisEPLParser.Node_listContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#node_list.
    def exitNode_list(self, ctx:GnosisEPLParser.Node_listContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#node.
    def enterNode(self, ctx:GnosisEPLParser.NodeContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#node.
    def exitNode(self, ctx:GnosisEPLParser.NodeContext):
        pass


    # Enter a parse tree produced by GnosisEPLParser#aggregator.
    def enterAggregator(self, ctx:GnosisEPLParser.AggregatorContext):
        pass

    # Exit a parse tree produced by GnosisEPLParser#aggregator.
    def exitAggregator(self, ctx:GnosisEPLParser.AggregatorContext):
        pass



del GnosisEPLParser


from .GnosisEPLListener import GnosisEPLListener


class GnosisEPLParserException(RuntimeError):
    """
    Exception raised by any error during the parse of the query
    """

    def __init__(self, node):
        payload = node.getPayload()
        symbol = node.getText()
        message = "Error parsing query. '{}' is not valid in the line {}:{}".format(
            symbol,
            payload.line,
            payload.column,
            payload
        )
        self.message = message
        self.node = node
        super(GnosisEPLParserException, self).__init__(message)


class ToDictionaryEPLListener(GnosisEPLListener):

    def __init__(self):
        self.query = {}

    def int_or_str_cast(self, value):
        try:
            return int(value)
        except ValueError:
            return value

    def enterWindow(self, ctx):
        args = [self.int_or_str_cast(arg.getText()) for arg in ctx.window_arg_list().window_arg()]
        window = {
            'window_type': ctx.window_type().getText(),
            'args': args
        }
        self.query['window'] = window

    def enterQuery_name(self, ctx):
        self.query['name'] = ctx.getText()

    def enterOutput_list(self, ctx):
        self.query['output'] = [p.getText() for p in ctx.output()]

    def enterContent(self, ctx):
        self.query['content'] = [c.getText() for c in ctx.content_service()]

    def enterPublisher_list(self, ctx):
        self.query['from'] = [p.getText() for p in ctx.publisher()]

    def enterMatch_clause(self, ctx):
        match = f'MATCH {ctx.getText()}'
        is_optional = 'match' in self.query.keys()
        key = 'match'
        if is_optional:
            match = f'OPTIONAL {match}'
            key = 'optional_match'
        self.query[key] = match

    def enterWhere_clause(self, ctx):
        self.query['where'] = f'WHERE {ctx.getText()}'

    def enterQos_metric_list(self, ctx):
        self.query['qos_policies'] = {}

    def enterQos_metric(self, ctx):
        name = ctx.qos_metric_name().getText()
        attr_value_ctx = ctx.qos_metric_value().attribute_value()

        num_val = attr_value_ctx.attribute_value_num()
        if num_val:
            val = float(num_val.getText())
        else:
            str_val_list = attr_value_ctx.attribute_value_str().attribute_value_str_inner()
            val = ''.join([v.getText() for v in str_val_list])
        self.query['qos_policies'][name] = val

    def enterNode_list(self, ctx):
        self.query['ret'] = f'RETURN {ctx.getText()}'

    def visitErrorNode(self, node):
        raise GnosisEPLParserException(node)



import unittest
from gnosis_epl.main import QueryParser
from gnosis_epl.grammar.epl_listener import GnosisEPLParserException


class TestGnosisEPLGrammar(unittest.TestCase):

    def setUp(self):
        self.parser = QueryParser()

    def tearDown(self):
        pass

    def test_query_name_is_correctly_parsed(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection, SpeachToText, TextLanguageModel
            MATCH (c1:Car {color:'blue'}) , (c2:Car {color:'white'})
            FROM test
            WITHIN TUMBLING_COUNT_WINDOW(2)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        self.assertIn('name', query_dict)
        self.assertEqual(query_dict['name'], 'my_first_query')

    def test_query_name_with_num_is_correctly_parsed(self):
        query_text = """REGISTER QUERY my_first_query000
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c1:Car {color:'blue'}) , (c2:Car {color:'white'})
            FROM test
            WITHIN TUMBLING_COUNT_WINDOW(2)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        self.assertIn('name', query_dict)
        self.assertEqual(query_dict['name'], 'my_first_query000')

    def test_query_name_exception_when_invalid(self):
        query_text = """REGISTER QUERY Invalid Query Name
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c1:Car {color:'blue'}) , (c2:Car {color:'white'})
            FROM test
            WITHIN TUMBLING_COUNT_WINDOW(2)
            RETURN *"""

        with self.assertRaises(GnosisEPLParserException):
            query_dict = self.parser.parse(query_text)

    def test_output_with_single_output(self):
        query_text = """REGISTER QUERY my_first_query000
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c1:Car {color:'blue'}) , (c2:Car {color:'white'})
            FROM test
            WITHIN TUMBLING_COUNT_WINDOW(2)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        self.assertIn('output', query_dict)
        self.assertListEqual(query_dict['output'], ['K_GRAPH_JSON'])

    def test_output_with_multiple_output(self):
        query_text = """REGISTER QUERY my_first_query000
            OUTPUT K_GRAPH_JSON, VIDEO_STREAM
            CONTENT ObjectDetection, ColorDetection
            MATCH (c1:Car {color:'blue'}) , (c2:Car {color:'white'})
            FROM test
            WITHIN TUMBLING_COUNT_WINDOW(2)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        self.assertIn('output', query_dict)
        self.assertListEqual(query_dict['output'], ['K_GRAPH_JSON', 'VIDEO_STREAM'])

    def test_from_is_correctly_parsed_when_single_value(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c1:Car {color:'blue'}) , (c2:Car {color:'white'})
            FROM test
            WITHIN TUMBLING_COUNT_WINDOW(2)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        self.assertIn('from', query_dict)
        self.assertEqual(query_dict['from'], ['test'])

    def test_from_is_correctly_parsed_when_multi_value(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c1:Car {color:'blue'}) , (c2:Car {color:'white'})
            FROM test, test2
            WITHIN TUMBLING_COUNT_WINDOW(2)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        self.assertIn('from', query_dict)
        self.assertEqual(query_dict['from'], ['test', 'test2'])

    def test_from_is_correctly_parsed_when_asterisk_value(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c1:Car {color:'blue'}) , (c2:Car {color:'white'})
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(2)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        self.assertIn('from', query_dict)
        self.assertEqual(query_dict['from'], ['*'])

    def test_content_is_correctly_parsed_when_single_value(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection
            MATCH (c1:Car {color:'blue'}) , (c2:Car {color:'white'})
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(2)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        self.assertIn('content', query_dict)
        self.assertEqual(query_dict['content'], ['ObjectDetection'])

    def test_content_is_correctly_parsed_when_multiple_values(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c1:Car {color:'blue'}) , (c2:Car {color:'white'})
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(2)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        self.assertIn('content', query_dict)
        self.assertEqual(query_dict['content'], ['ObjectDetection', 'ColorDetection'])

    def test_content_is_correctly_parsed_when_not_present(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            MATCH (c1:Car {color:'blue'}) , (c2:Car {color:'white'})
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(2)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        self.assertNotIn('content', query_dict)

    def test_window_is_correctly_parsed_with_single_arg(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c1:Car {color:'blue'}) , (c2:Car {color:'white'})
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(2)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_dict = {
            'window_type': 'TUMBLING_COUNT_WINDOW',
            'args': [2]
        }
        self.assertIn('window', query_dict)
        self.assertEqual(query_dict['window'], excepted_dict)

    def test_window_is_correctly_parsed_with_multiple_arg(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c1:Car {color:'blue'}) , (c2:Car {color:'white'})
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(2, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_dict = {
            'window_type': 'TUMBLING_COUNT_WINDOW',
            'args': [2, 1]
        }
        self.assertIn('window', query_dict)
        self.assertEqual(query_dict['window'], excepted_dict)

    def test_window_is_correctly_parsed_with_mixed_type_arg(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c1:Car {color:'blue'}) , (c2:Car {color:'white'})
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_dict = {
            'window_type': 'TUMBLING_COUNT_WINDOW',
            'args': ['a1b2c', 1]
        }
        self.assertIn('window', query_dict)
        self.assertEqual(query_dict['window'], excepted_dict)

    def test_entire_match_with_is_correctly_parsed_when_obj_not_defined(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (:Car)
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (:Car)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)

    def test_entire_match_with_is_correctly_parsed_when_obj_is_defined(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (c:Car)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)

    def test_entire_match_with_is_correctly_parsed_when_non_related_matches(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (:Car), (:Person)
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (:Car), (:Person)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)

    def test_entire_match_with_is_correctly_parsed_when_single_left_relation(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (:Car)-->(:Person)
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (:Car)-->(:Person)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)

    def test_entire_match_with_is_correctly_parsed_when_single_right_relation(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (:Car)<--(:Person)
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (:Car)<--(:Person)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)

    def test_entire_match_with_is_correctly_parsed_when_single_gen_relation(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (:Car)--(:Person)
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (:Car)--(:Person)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)

    def test_entire_match_with_is_correctly_parsed_when_single_relation_detailed(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (:Car)-[:something]-(:Person)
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (:Car)-[:something]-(:Person)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)

    def test_entire_match_with_is_correctly_parsed_when_single_relation_detailed_label(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (:Car)-[r:something]-(:Person)
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (:Car)-[r:something]-(:Person)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)

    def test_entire_match_with_is_correctly_parsed_when_single_relation_detailed_attrs(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (:Car)-[:something {something:123, abc:"test"}]-(:Person)
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (:Car)-[:something {something:123, abc:"test"}]-(:Person)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)

    def test_entire_match_with_is_correctly_parsed_when_single_relation_detailed_attrs_label_node(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)-[:something {something:123, abc:"test"}]-(p:Person)
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (c:Car)-[:something {something:123, abc:"test"}]-(p:Person)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)

    def test_entire_match_with_is_correctly_parsed_when_single_relation_detailed_attrs_label_node_directed_right(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)-[:something {something:123, abc:"test"}]->(p:Person)
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (c:Car)-[:something {something:123, abc:"test"}]->(p:Person)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)

    def test_entire_match_with_is_correctly_parsed_when_single_relation_detailed_attrs_label_node_directed_left(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)<-[:something {something:123, abc:"test"}]-(p:Person)
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (c:Car)<-[:something {something:123, abc:"test"}]-(p:Person)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)

    def test_entire_match_with_is_correctly_parsed_when_multiple_relation_gen(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)--(p:Person)--(ct:Cat)
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (c:Car)--(p:Person)--(ct:Cat)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)

    def test_entire_match_with_is_correctly_parsed_when_multiple_relation_attrs_label_directed(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)-->(p:Person)<-[r:Owns]-(ct:Cat)
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (c:Car)-->(p:Person)<-[r:Owns]-(ct:Cat)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)

    def test_entire_optional_match_with_is_correctly_parsed_when_multiple_relation_attrs_label_directed(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)-->(p:Person)<-[r:Owns]-(ct:Cat)<--(p)
            OPTIONAL MATCH (c)-->(x)
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (c:Car)-->(p:Person)<-[r:Owns]-(ct:Cat)<--(p)'
        excepted_optional_str = 'OPTIONAL MATCH (c)-->(x)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)
        self.assertIn('optional_match', query_dict)
        self.assertEqual(query_dict['optional_match'], excepted_optional_str)

    def test_entire_where_single(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)-->(p:Person)<-[r:Owns]-(ct:Cat)<--(p)
            OPTIONAL MATCH (c)-->(x)
            WHERE c.color = "blue"
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'WHERE c.color = "blue"'
        self.assertIn('where', query_dict)
        self.assertEqual(query_dict['where'], excepted_str)

    def test_entire_where_multiple_with_and(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)-->(p:Person)<-[r:Owns]-(ct:Cat)<--(p)
            OPTIONAL MATCH (c)-->(x)
            WHERE c.color = "blue" AND p.age = 23
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'WHERE c.color = "blue" AND p.age = 23'
        self.assertIn('where', query_dict)
        self.assertEqual(query_dict['where'], excepted_str)

    def test_entire_where_multiple_with_or(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)-->(p:Person)<-[r:Owns]-(ct:Cat)<--(p)
            OPTIONAL MATCH (c)-->(x)
            WHERE c.color = "blue" OR p.age = 23
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'WHERE c.color = "blue" OR p.age = 23'
        self.assertIn('where', query_dict)
        self.assertEqual(query_dict['where'], excepted_str)

    def test_entire_where_single_lesser_than(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)-->(p:Person)<-[r:Owns]-(ct:Cat)<--(p)
            OPTIONAL MATCH (c)-->(x)
            WHERE p.age < 23
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'WHERE p.age < 23'
        self.assertIn('where', query_dict)
        self.assertEqual(query_dict['where'], excepted_str)

    def test_entire_where_single_lesser_than_eq(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)-->(p:Person)<-[r:Owns]-(ct:Cat)<--(p)
            OPTIONAL MATCH (c)-->(x)
            WHERE p.age <= 23
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'WHERE p.age <= 23'
        self.assertIn('where', query_dict)
        self.assertEqual(query_dict['where'], excepted_str)

    def test_entire_where_single_greater_than(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)-->(p:Person)<-[r:Owns]-(ct:Cat)<--(p)
            OPTIONAL MATCH (c)-->(x)
            WHERE p.age > 23
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'WHERE p.age > 23'
        self.assertIn('where', query_dict)
        self.assertEqual(query_dict['where'], excepted_str)

    def test_entire_where_single_greater_than_eq(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)-->(p:Person)<-[r:Owns]-(ct:Cat)<--(p)
            OPTIONAL MATCH (c)-->(x)
            WHERE p.age >= 23
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'WHERE p.age >= 23'
        self.assertIn('where', query_dict)
        self.assertEqual(query_dict['where'], excepted_str)

    def test_entire_where_single_diff(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)-->(p:Person)<-[r:Owns]-(ct:Cat)<--(p)
            OPTIONAL MATCH (c)-->(x)
            WHERE p.age <> 23
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'WHERE p.age <> 23'
        self.assertIn('where', query_dict)
        self.assertEqual(query_dict['where'], excepted_str)

    def test_entire_where_compare_nodes(self):
        query_text = """REGISTER QUERY my_first_query
            OUTPUT K_GRAPH_JSON
            CONTENT ObjectDetection, ColorDetection
            MATCH (c:Car)-->(p1:Person)<-[r:Owns]-(ct:Cat)<--(p)
            OPTIONAL MATCH (c)-->(x)
            WHERE p1.age <> ct.age
            FROM *
            WITHIN TUMBLING_COUNT_WINDOW(a1b2c, 1)
            RETURN *"""
        query_dict = self.parser.parse(query_text)
        excepted_str = 'WHERE p1.age <> ct.age'
        self.assertIn('where', query_dict)
        self.assertEqual(query_dict['where'], excepted_str)

    def test_simple_query(self):
        query_text = (
            "REGISTER QUERY my_first_query "
            "OUTPUT K_GRAPH_JSON "
            "CONTENT ObjectDetection "
            "MATCH (c:Car)--(p:Person) "
            "FROM publisher1 "
            "WITHIN TUMBLING_COUNT_WINDOW(1) "
            "RETURN *"
        )
        query_dict = self.parser.parse(query_text)
        excepted_str = 'MATCH (c:Car)--(p:Person)'
        self.assertIn('match', query_dict)
        self.assertEqual(query_dict['match'], excepted_str)

    def test_entire_ret_simple(self):
        query_text = (
            "REGISTER QUERY my_first_query "
            "OUTPUT K_GRAPH_JSON "
            "CONTENT ObjectDetection "
            "MATCH (c:Car)--(p:Person) "
            "FROM publisher1 "
            "WITHIN TUMBLING_COUNT_WINDOW(1) "
            "RETURN c"
        )
        query_dict = self.parser.parse(query_text)
        excepted_str = 'RETURN c'
        self.assertIn('ret', query_dict)
        self.assertEqual(query_dict['ret'], excepted_str)

    def test_entire_ret_simple_attr(self):
        query_text = (
            "REGISTER QUERY my_first_query "
            "OUTPUT K_GRAPH_JSON "
            "CONTENT ObjectDetection "
            "MATCH (c:Car)--(p:Person) "
            "FROM publisher1 "
            "WITHIN TUMBLING_COUNT_WINDOW(1) "
            "RETURN c.color"
        )
        query_dict = self.parser.parse(query_text)
        excepted_str = 'RETURN c.color'
        self.assertIn('ret', query_dict)
        self.assertEqual(query_dict['ret'], excepted_str)

    def test_entire_ret_mult_attr(self):
        query_text = (
            "REGISTER QUERY my_first_query "
            "OUTPUT K_GRAPH_JSON "
            "CONTENT ObjectDetection "
            "MATCH (c:Car)--(p:Person) "
            "FROM publisher1 "
            "WITHIN TUMBLING_COUNT_WINDOW(1) "
            "RETURN c.color, p"
        )
        query_dict = self.parser.parse(query_text)
        excepted_str = 'RETURN c.color, p'
        self.assertIn('ret', query_dict)
        self.assertEqual(query_dict['ret'], excepted_str)

    def test_entire_ret_mult_attr2(self):
        query_text = (
            "REGISTER QUERY my_first_query "
            "OUTPUT K_GRAPH_JSON "
            "CONTENT ObjectDetection "
            "MATCH (c:Car)--(p:Person) "
            "FROM publisher1 "
            "WITHIN TUMBLING_COUNT_WINDOW(1) "
            "RETURN c.color, p.name"
        )
        query_dict = self.parser.parse(query_text)
        excepted_str = 'RETURN c.color, p.name'
        self.assertIn('ret', query_dict)
        self.assertEqual(query_dict['ret'], excepted_str)

    def test_entire_ret_asterisk(self):
        query_text = (
            "REGISTER QUERY my_first_query "
            "OUTPUT K_GRAPH_JSON "
            "CONTENT ObjectDetection "
            "MATCH (c:Car)--(p:Person) "
            "FROM publisher1 "
            "WITHIN TUMBLING_COUNT_WINDOW(1) "
            "RETURN *"
        )
        query_dict = self.parser.parse(query_text)
        excepted_str = 'RETURN *'
        self.assertIn('ret', query_dict)
        self.assertEqual(query_dict['ret'], excepted_str)

    def test_entire_ret_complex_aggr(self):
        query_text = (
            "REGISTER QUERY my_first_query "
            "OUTPUT K_GRAPH_JSON "
            "CONTENT ObjectDetection "
            "MATCH (c:Car)--(p:Person) "
            "FROM publisher1 "
            "WITHIN TUMBLING_COUNT_WINDOW(1) "
            "RETURN count(distinct p) as PersonCount, p as People"
        )
        query_dict = self.parser.parse(query_text)
        excepted_str = 'RETURN count(distinct p) as PersonCount, p as People'
        self.assertIn('ret', query_dict)
        self.assertEqual(query_dict['ret'], excepted_str)

    def test_entire_where_with_list_attributes(self):
        query_text = (
            "REGISTER QUERY my_first_query "
            "OUTPUT K_GRAPH_JSON "
            "CONTENT ObjectDetection "
            "MATCH (p:person), (p2:person) "
            "WHERE p.bounding_box[2] < p2.bounding_box[0] "
            "FROM publisher1 "
            "WITHIN TUMBLING_COUNT_WINDOW(1) "
            "RETURN *"
        )
        query_dict = self.parser.parse(query_text)
        excepted_str = 'WHERE p.bounding_box[2] < p2.bounding_box[0]'
        self.assertIn('where', query_dict)
        self.assertEqual(query_dict['where'], excepted_str)

    def test_parsed_with_qos_statements_single_metric_num(self):
        query_text = (
            "REGISTER QUERY my_first_query "
            "OUTPUT K_GRAPH_JSON "
            "CONTENT ObjectDetection "
            "MATCH (p:person), (p2:person) "
            "WHERE p.bounding_box[2] < p2.bounding_box[0] "
            "FROM publisher1 "
            "WITHIN TUMBLING_COUNT_WINDOW(1) "
            "WITH_QOS accuracy = 123 "
            "RETURN *"
        )
        query_dict = self.parser.parse(query_text)
        expected_dict = {
            'accuracy': 123,
        }
        self.assertIn('qos_policies', query_dict)
        self.assertDictEqual(query_dict['qos_policies'], expected_dict)

    def test_parsed_with_qos_statements_single_metric_str(self):
        query_text = (
            "REGISTER QUERY my_first_query "
            "OUTPUT K_GRAPH_JSON "
            "CONTENT ObjectDetection "
            "MATCH (p:person), (p2:person) "
            "WHERE p.bounding_box[2] < p2.bounding_box[0] "
            "FROM publisher1 "
            "WITHIN TUMBLING_COUNT_WINDOW(1) "
            "WITH_QOS accuracy = \"min mean max\" "
            "RETURN *"
        )
        query_dict = self.parser.parse(query_text)
        expected_dict = {
            'accuracy': 'min mean max',
        }
        self.assertIn('qos_policies', query_dict)
        self.assertDictEqual(query_dict['qos_policies'], expected_dict)

    def test_parsed_with_qos_statements_multiple_metrics(self):
        query_text = (
            "REGISTER QUERY my_first_query "
            "OUTPUT K_GRAPH_JSON "
            "CONTENT ObjectDetection "
            "MATCH (p:person), (p2:person) "
            "WHERE p.bounding_box[2] < p2.bounding_box[0] "
            "FROM publisher1 "
            "WITHIN TUMBLING_COUNT_WINDOW(1) "
            "WITH_QOS accuracy = \"min mean max\", energy = 123, latency = 'average' "
            "RETURN *"
        )
        query_dict = self.parser.parse(query_text)
        expected_dict = {
            'accuracy': 'min mean max',
            'energy': 123.0,
            'latency': 'average'
        }
        self.assertIn('qos_policies', query_dict)
        self.assertDictEqual(query_dict['qos_policies'], expected_dict)



from antlr4 import InputStream, CommonTokenStream, ParseTreeWalker

from .grammar.GnosisEPLLexer import GnosisEPLLexer
from .grammar.GnosisEPLParser import GnosisEPLParser
from .grammar.epl_listener import ToDictionaryEPLListener


class QueryParser():

    def parse(self, query_text):
        lexer = GnosisEPLLexer(InputStream(query_text))
        stream = CommonTokenStream(lexer)
        parser = GnosisEPLParser(stream)
        tree = parser.query()

        listener = ToDictionaryEPLListener()
        walker = ParseTreeWalker()
        walker.walk(listener, tree)
        return listener.query
